realTerrainManager = {};

addModEventListener(realTerrainManager);

function realTerrainManager:loadMap(name)
	if not g_isRealTerrainManagerLoaded then
		addConsoleCommand("RTDebugRender", "Toggle debug rendering.", "toggleDebugRender", self);
		addConsoleCommand("RTPrintWheelInfo", "Print Wheel Infoes.", "printWheelInfo", self);

		g_isRealTerrainManagerLoaded = true;
	end;
end;

function realTerrainManager:deleteMap()
	if g_isRealTerrainManagerLoaded then
		removeConsoleCommand("RTDebugRender");
		removeConsoleCommand("RTPrintWheelInfo");

		g_isRealTerrainManagerLoaded = false;
	end;
end;

function realTerrainManager:update(dt) end;

function realTerrainManager:draw() end;

function realTerrainManager:mouseEvent(posX, posY, isDown, isUp, mouseKey) end;

function realTerrainManager:keyEvent() end;

function realTerrainManager:toggleDebugRender()
	if realTerrain then
		realTerrain.printDebug = not realTerrain.printDebug;
		return "Real Terrain Debug Render Mode: " .. (realTerrain.printDebug and "ON" or "OFF");
	end;
end;

function realTerrainManager:printWheelInfo()
	if realTerrain then
		realTerrain.printWheelInfo = true;
		return "Real Terrain Printing Wheel Infoes!";
	end;
end;

function realTerrainManager:tableShow(t, name, indent, maxDepth)
	local cart; -- a container
	local autoref; -- for self references
	maxDepth = maxDepth or 50;
	local depth = 0;

	-- (RiciLake) returns true if the table is empty
	local function isemptytable(t)
		return next(t) == nil;
	end;

	local function basicSerialize(o)
		local so = tostring(o);
		local oType = type(o);
		if oType == 'function' then
			local info = debug.getinfo(o, 'S')
			-- info.name is nil because o is not a calling level
			if info.what == 'C' then
				return ('"%s, C function"'):format(so);
			else
				-- the information is defined in a script
				return ('"%s, defined in %s (lines %d-%d)"'):format(so, info.source, info.linedefined, info.lastlinedefined);
			end
		elseif oType == 'number' then
			return so;
		elseif oType == 'boolean' then
			return ('%s'):format(so);
		else
			return ('%q'):format(so);
		end;
	end;

	local function addToCart(value, name, indent, saved, field, curDepth)
		indent = indent or ''
		saved = saved or {}
		field = field or name
		-- cart = cart .. indent .. field
		cart = indent .. field
		-- print(('addToCart(value=%q, name=%q, indent, saved, field=%q, curDepth=%d)'):format(tostring(value), tostring(name), tostring(field), tostring(curDepth)));

		if type(value) ~= 'table' then
			cart = cart .. ' = ' .. basicSerialize(value) .. ';';
			print(cart);
		else
			if saved[value] then
				cart = cart .. ' = {}; -- ' .. saved[value] .. ' (self reference)';
				print(cart);
				autoref = autoref .. name .. ' = ' .. saved[value] .. ';\n';
			else
				saved[value] = name;
				if isemptytable(value) then
					cart = cart .. ' = {};';
					print(cart);
				else
					if curDepth <= maxDepth then
						cart = cart .. ' = {';
						print(cart);
						for k, v in pairs(value) do
							k = basicSerialize(k);
							local fname = string.format('%s[%s]', name, k);
							field = string.format('[%s]', k);
							-- three spaces between levels
							addToCart(v, fname, indent .. '\t', saved, field, curDepth + 1);
						end;
						cart = indent .. '};';
						print(cart);
					else
						cart = cart .. ' = { ... };';
						print(cart);
					end;
				end;
			end;
		end;
	end;

	name = name or '__unnamed__';
	if type(t) ~= 'table' then
		return name .. ' = ' .. basicSerialize(t);
	end;
	cart, autoref = '', '';
	addToCart(t, name, indent, nil, nil, depth + 1)
	-- return cart .. autoref
	print(autoref);
	return ('-- %s %s -END- %s --'):format(('#'):rep(40), name, ('#'):rep(40));
end;